#!/usr/bin/env python

import compiler

def process_nodes(root_node, prefix):

    """
    Under the 'root_node', process all suitable expression nodes which employ
    Name nodes using the given 'prefix'.
    """

    for node in root_node.getChildNodes():
        # Identify suitable names and add such nodes
        if isinstance(node, compiler.ast.CallFunc):
            process_callfunc(node, prefix)
        elif isinstance(node, compiler.ast.Getattr):
            process_getattr(node, prefix, root_node)
        else:
            process_nodes(node, prefix)

def process_callfunc(node, prefix):
    # Check the target.
    target = node.node
    if isinstance(target, compiler.ast.Getattr):
        process_getattr(target, prefix, node)
    else:
        process_nodes(node, prefix)

def process_getattr(node, prefix, parent):
    # Check the target.
    target = node.expr
    if isinstance(target, compiler.ast.Name) and target.name.startswith(prefix):

        # Replace CallFunc plus Getattr occurrences:
        # node.attr(args) -> Node_attr(node, args)

        if isinstance(parent, compiler.ast.CallFunc):
            parent.node = compiler.ast.Name("Node_%s" % node.attrname)
            parent.args.insert(0, compiler.ast.Name(target.name))

        # Replace plain Getattr nodes:
        # node.attr -> Node_attr(node)
        # NOTE: Nasty but necessary rewiring of the parent node required.

        else:
            replacement = compiler.ast.CallFunc(
                compiler.ast.Name("Node_%s" % node.attrname),
                [node.expr]
                )
            for key, value in parent.__dict__.items():
                # Detect lists.
                try:
                    if node in value:
                        index = value.index(node)
                        value[index] = replacement
                except TypeError:
                    if value is node:
                        parent.__dict__[key] = replacement
    else:
        process_nodes(node, prefix)

def process_file(filename, prefix="x2_"):

    """
    Process the module given by the specified 'filename'. The optional special
    'prefix' marks those variables to be processed.
    """

    # Open the module as an AST.

    module = compiler.parseFile(filename)

    # Find references to special variables.

    process_nodes(module, prefix)
    return module

# vim: tabstop=4 expandtab shiftwidth=4
